@@ -8,7 +8,7 @@ import time
 from warnings import warn
 from types import MethodType
 from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, TypeVar, Union, get_type_hints
-from typing_inspect import is_literal_type, get_args, get_origin, is_union_type
+from typing_inspect import is_literal_type, get_args
 
 from tap.utils import (
     get_class_variables,
@@ -16,6 +16,7 @@ from tap.utils import (
     get_git_root,
     get_dest,
     get_git_url,
+    get_origin,
     has_git,
     has_uncommitted_changes,
     is_option_arg,
@@ -32,16 +33,10 @@ from tap.utils import (
 
 # Constants
 EMPTY_TYPE = get_args(List)[0] if len(get_args(List)) > 0 else tuple()
+BOXED_COLLECTION_TYPES = {List, list, Set, set, Tuple, tuple}
+OPTIONAL_TYPES = {Optional, Union}
+BOXED_TYPES = BOXED_COLLECTION_TYPES | OPTIONAL_TYPES
 
-SUPPORTED_DEFAULT_BASE_TYPES = {str, int, float, bool}
-SUPPORTED_DEFAULT_OPTIONAL_TYPES = {Optional, Optional[str], Optional[int], Optional[float], Optional[bool]}
-SUPPORTED_DEFAULT_LIST_TYPES = {List, List[str], List[int], List[float], List[bool]}
-SUPPORTED_DEFAULT_SET_TYPES = {Set, Set[str], Set[int], Set[float], Set[bool]}
-SUPPORTED_DEFAULT_COLLECTION_TYPES = SUPPORTED_DEFAULT_LIST_TYPES | SUPPORTED_DEFAULT_SET_TYPES | {Tuple}
-SUPPORTED_DEFAULT_BOXED_TYPES = SUPPORTED_DEFAULT_OPTIONAL_TYPES | SUPPORTED_DEFAULT_COLLECTION_TYPES
-SUPPORTED_DEFAULT_TYPES = set.union(SUPPORTED_DEFAULT_BASE_TYPES,
-                                    SUPPORTED_DEFAULT_OPTIONAL_TYPES,
-                                    SUPPORTED_DEFAULT_COLLECTION_TYPES)
 
 TapType = TypeVar('TapType', bound='Tap')
 
@@ -104,7 +99,7 @@ class Tap(ArgumentParser):
         # Load in the configuration files
         self.args_from_configs = self._load_from_config_files(config_files)
 
-        # Perform additional configuration such as modifying add_arguments or adding subparsers
+        # Perform additional configuration such as adding arguments or adding subparsers
         self._configure()
 
         # Indicate that initialization is complete
@@ -125,6 +120,9 @@ class Tap(ArgumentParser):
         :param name_or_flags: Either a name or a list of option strings, e.g. foo or -f, --foo.
         :param kwargs: Keyword arguments.
         """
+        # Set explicit bool
+        explicit_bool = self._explicit_bool
+
         # Get variable name
         variable = get_argument_name(*name_or_flags)
 
@@ -168,6 +166,21 @@ class Tap(ArgumentParser):
 
             # If type is not explicitly provided, set it if it's one of our supported default types
             if 'type' not in kwargs:
+
+                # Unbox Optional[type] and set var_type = type
+                if get_origin(var_type) in OPTIONAL_TYPES:
+                    var_args = get_args(var_type)
+
+                    if len(var_args) > 0:
+                        var_type = get_args(var_type)[0]
+
+                        # If var_type is tuple as in Python 3.6, change to a typing type
+                        # (e.g., (typing.List, <class 'bool'>) ==> typing.List[bool])
+                        if isinstance(var_type, tuple):
+                            var_type = var_type[0][var_type[1:]]
+
+                        explicit_bool = True
+
                 # First check whether it is a literal type or a boxed literal type
                 if is_literal_type(var_type):
                     var_type, kwargs['choices'] = get_literals(var_type, variable)
@@ -195,27 +208,10 @@ class Tap(ArgumentParser):
                         kwargs['nargs'] = len(types)
 
                     var_type = TupleTypeEnforcer(types=types, loop=loop)
-                # To identify an Optional type, check if it's a union of a None and something else
-                elif (
-                    is_union_type(var_type)
-                    and len(get_args(var_type)) == 2
-                    and isinstance(None, get_args(var_type)[1])
-                    and is_literal_type(get_args(var_type)[0])
-                ):
-                    var_type, kwargs['choices'] = get_literals(get_args(var_type)[0], variable)
-                elif var_type not in SUPPORTED_DEFAULT_TYPES:
-                    is_required = kwargs.get('required', False)
-                    arg_params = 'required=True' if is_required else f'default={getattr(self, variable)}'
-                    raise ValueError(
-                        f'Variable "{variable}" has type "{var_type}" which is not supported by default.\n'
-                        f'Please explicitly add the argument to the parser by writing:\n\n'
-                        f'def add_arguments(self) -> None:\n'
-                        f'    self.add_argument("--{variable}", type=func, {arg_params})\n\n'
-                        f'where "func" maps from str to {var_type}.')
 
-                if var_type in SUPPORTED_DEFAULT_BOXED_TYPES:
+                if get_origin(var_type) in BOXED_TYPES:
                     # If List or Set type, set nargs
-                    if (var_type in SUPPORTED_DEFAULT_COLLECTION_TYPES
+                    if (get_origin(var_type) in BOXED_COLLECTION_TYPES
                             and kwargs.get('action') not in {'append', 'append_const'}):
                         kwargs['nargs'] = kwargs.get('nargs', '*')
 
@@ -228,13 +224,12 @@ class Tap(ArgumentParser):
                     else:
                         var_type = arg_types[0]
 
-                    # Handle the cases of Optional[bool], List[bool], Set[bool]
+                    # Handle the cases of List[bool], Set[bool], Tuple[bool]
                     if var_type == bool:
                         var_type = boolean_type
-
                 # If bool then set action, otherwise set type
                 if var_type == bool:
-                    if self._explicit_bool:
+                    if explicit_bool:
                         kwargs['type'] = boolean_type
                         kwargs['choices'] = [True, False]  # this makes the help message more helpful
                     else:
@@ -253,9 +248,9 @@ class Tap(ArgumentParser):
         if self._initialized:
             raise ValueError('add_argument cannot be called after initialization. '
                              'Arguments must be added either as class variables or by overriding '
-                             'add_arguments and including a self.add_argument call there.')
+                             'configure and including a self.add_argument call there.')
 
-        variable = get_argument_name(*name_or_flags)
+        variable = get_argument_name(*name_or_flags).replace('-', '_')
         self.argument_buffer[variable] = (name_or_flags, kwargs)
 
     def _add_arguments(self) -> None:
@@ -268,7 +263,7 @@ class Tap(ArgumentParser):
             else:
                 self._add_argument(f'--{variable}')
 
-        # Add any arguments that were added manually in add_arguments but aren't class variables (in order)
+        # Add any arguments that were added manually in configure but aren't class variables (in order)
         for variable, (name_or_flags, kwargs) in self.argument_buffer.items():
             if variable not in self.class_variables:
                 self._add_argument(*name_or_flags, **kwargs)
@@ -404,6 +399,15 @@ class Tap(ArgumentParser):
                 if type(value) == list:
                     var_type = get_origin(self._annotations[variable])
 
+                    # Unpack nested boxed types such as Optional[List[int]]
+                    if var_type is Union:
+                        var_type = get_origin(get_args(self._annotations[variable])[0])
+
+                        # If var_type is tuple as in Python 3.6, change to a typing type
+                        # (e.g., (typing.Tuple, <class 'bool'>) ==> typing.Tuple)
+                        if isinstance(var_type, tuple):
+                            var_type = var_type[0]
+
                     if var_type in (Set, set):
                         value = set(value)
                     elif var_type in (Tuple, tuple):
