@@ -24,7 +24,7 @@ from typing import (
     Union,
 )
 from typing_extensions import Literal
-from typing_inspect import get_args
+from typing_inspect import get_args, get_origin as typing_inspect_get_origin
 
 
 NO_CHANGES_STATUS = """nothing to commit, working tree clean"""
@@ -90,7 +90,7 @@ def get_git_url(commit_hash: bool = True) -> str:
 
     if commit_hash:
         # Add tree and hash of current commit
-        url = os.path.join(url, 'tree', get_git_hash())
+        url = f'{url}/tree/{get_git_hash()}'
 
     return url
 
@@ -276,7 +276,7 @@ def boolean_type(flag_value: str) -> bool:
         return True
     if 'false'.startswith(flag_value.lower()) or flag_value == '0':
         return False
-    raise ArgumentTypeError(f'Value has to be a prefix of "True" or "False" (case insensitive) or "1" or "0".')
+    raise ArgumentTypeError('Value has to be a prefix of "True" or "False" (case insensitive) or "1" or "0".')
 
 
 class TupleTypeEnforcer:
@@ -331,7 +331,7 @@ def _nested_replace_type(obj: Any, find_type: type, replace_type: type) -> Any:
     return obj
 
 
-def define_python_object_encoder(skip_unpicklable: bool = False) -> 'PythonObjectEncoder':
+def define_python_object_encoder(skip_unpicklable: bool = False) -> 'PythonObjectEncoder':  # noqa F821
 
     class PythonObjectEncoder(JSONEncoder):
         """Stores parameters that are not JSON serializable as pickle dumps.
@@ -467,3 +467,16 @@ def enforce_reproducibility(saved_reproducibility_data: Optional[Dict[str, str]]
     if current_reproducibility_data['git_has_uncommitted_changes']:
         raise ValueError(f'{no_reproducibility_message}: Uncommitted changes '
                          f'in current args.')
+
+
+# TODO: remove this once typing_inspect.get_origin is fixed for Python 3.8 and 3.9
+# https://github.com/ilevkivskyi/typing_inspect/issues/64
+# https://github.com/ilevkivskyi/typing_inspect/issues/65
+def get_origin(tp: Any) -> Any:
+    """Same as typing_inspect.get_origin but fixes unparameterized generic types like Set."""
+    origin = typing_inspect_get_origin(tp)
+
+    if origin is None:
+        origin = tp
+
+    return origin
